from selve.communication import Command
from selve.protocol import *
from selve.commands import *

class CommeoServicePing(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.PING)

class CommeoServiceGetState(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.GETSTATE)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.status = ServiceState(int(methodResponse.parameters[0][1]))

class CommeoServiceGetVersion(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.GETVERSION)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.serial = str(methodResponse.parameters[0][1])
        self.version = int(methodResponse.parameters[1][1]) + "." + int(methodResponse.parameters[2][1]) + "." + int(methodResponse.parameters[3][1]) + "." + int(methodResponse.parameters[6][1])
        self.spec = int(methodResponse.parameters[4][1]) + "." + int(methodResponse.parameters[5][1])
        

class CommeoServiceReset(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.RESET)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoServiceFactoryReset(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.FACTORYRESET)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoServiceSetLed(Command):
    def __init__(self, ledMode):
        super().__init__(CommeoServiceCommand.SETLED, [(ParameterType.INT, ledMode)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoServiceGetLed(Command):
    def __init__(self):
        super().__init__(CommeoServiceCommand.GETLED)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ledmode = LEDMode(int(methodResponse.parameters[0][1]))
    
class CommeoParamSetForward(Command):
    def __init__(self, forwarding):
        super().__init__(CommeoParamCommand.SETFORWARD, [(ParameterType.INT, forwarding)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoParamGetForward(Command):
    def __init__(self):
        super().__init__(CommeoParamCommand.GETFORWARD)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.forwarding = Forwarding(int(methodResponse.parameters[0][1]))
    
class CommeoParamSetEvent(Command):
    def __init__(self, eventDevice, eventSensor, eventSender, eventLogging, eventDuty):
        super().__init__(CommeoParamCommand.SETEVENT, [(ParameterType.INT, eventDevice), (ParameterType.INT, eventSensor), (ParameterType.INT, eventSender), (ParameterType.INT, eventLogging), (ParameterType.INT, eventDuty)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
    
class CommeoParamGetEvent(Command):
    def __init__(self):
        super().__init__(CommeoParamCommand.GETEVENT)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.eventDevice = bool(methodResponse.parameters[0][1])
        self.eventSensor = bool(methodResponse.parameters[1][1])
        self.eventSender = bool(methodResponse.parameters[2][1])
        self.eventLogging = bool(methodResponse.parameters[3][1])
        self.eventDuty = bool(methodResponse.parameters[4][1])
    
class CommeoParamGetDuty(Command):
    def __init__(self):
        super().__init__(CommeoParamCommand.GETDUTY)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.dutyMode = DutyMode(int(methodResponse.parameters[0][1]))
        self.rfTraffic = int(methodResponse.parameters[1][1])
    
class CommeoParamGetRF(Command):
    def __init__(self):
        super().__init__(CommeoParamCommand.GETRF)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.netAddress = int(methodResponse.parameters[0][1])
        self.resetCount = int(methodResponse.parameters[1][1])
        self.rfBaseId = int(methodResponse.parameters[2][1])
        self.sensorNetAddress = int(methodResponse.parameters[3][1])
        self.rfSensorId = int(methodResponse.parameters[4][1])
        self.iveoResetCount = int(methodResponse.parameters[5][1])
        self.rfIveoId = int(methodResponse.parameters[6][1])
    


from os import wait
from selve.device import Device
from selve.communication import Command, CommandSingle
from selve.protocol import CommunicationType, DeviceClass, ScanState, SensorState, TeachState, lightDigital, rainDigital, tempDigital, windDigital
from selve.protocol import ParameterType
from selve.commands import CommeoSensorCommand
from selve.utils import singlemask
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
import logging

_LOGGER = logging.getLogger(__name__)

 
class CommeoSensorTeachStart(Command):
    def __init__(self):
        super().__init__(CommeoSensorCommand.TEACHSTART)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoSensorTeachStop(Command):
    def __init__(self):
        super().__init__(CommeoSensorCommand.TEACHSTOP)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
class CommeoSensorTeachResult(Command):
    def __init__(self):
        super().__init__(CommeoSensorCommand.TEACHRESULT)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.teachState = TeachState(int(methodResponse.parameters[0][1]))
        self.timeLeft = int(methodResponse.parameters[1][1])
        self.foundId = int(methodResponse.parameters[2][1])

class CommeoSensorGetIDs(Command):
    def __init__(self):
        super().__init__(CommeoSensorCommand.GETIDS)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class CommeoSensorGetInfo(Command):
    def __init__(self):
        super().__init__(CommeoSensorCommand.GETINFO)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.rfAddress = methodResponse.parameters[2][1]

class CommeoSensorGetValues(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoSensorCommand.GETVALUES, deviceId)


    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.windDigital = windDigital(int(methodResponse.parameters[1][1]))
        self.rainDigital = rainDigital(int(methodResponse.parameters[2][1]))
        self.tempDigital = tempDigital(int(methodResponse.parameters[3][1]))
        self.lightDigital = lightDigital(int(methodResponse.parameters[4][1]))
        self.sensorState = SensorState(int(methodResponse.parameters[5][1]))
        self.tempAnalog = int(methodResponse.parameters[6][1])
        self.windAnalog = int(methodResponse.parameters[7][1])
        self.sun1Analog = int(methodResponse.parameters[8][1])
        self.dayLightAnalog = int(methodResponse.parameters[9][1])
        self.sun2Analog = int(methodResponse.parameters[10][1])
        self.sun3Analog = int(methodResponse.parameters[11][1])
        

class CommeoSensorSetLabel(Command):
    def __init__(self, deviceId, name):
        super().__init__(CommeoSensorCommand.SETLABEL, [(ParameterType.INT, deviceId), (ParameterType.STRING, name)])

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSensorDelete(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoSensorCommand.DELETE, deviceId)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSensorWriteManual(Command):
    def __init__(self, deviceId, address, name):
        super().__init__(CommeoSensorCommand.WRITEMANUAL, [(ParameterType.INT, deviceId), (ParameterType.INT, address), (ParameterType.STRING, name)])
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class SensorDevice(Device):
    def __init__(self, gateway, id, discover = False):
        super().__init__(gateway, id, discover)
        self.communicationType = CommunicationType.COMMEO
        self.deviceClass = DeviceClass.SENSOR
        if discover:
            self.discover_properties()

    def discover_properties(self):
        try:
            command = CommeoSensorGetInfo(self.ID)
            command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.rfAddress = command.rfAddress
            self.state = command.state
            self.deviceClass = DeviceClass.SENSOR
            self.communicationType = CommunicationType.COMMEO
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))


    def scanSensors(self):
        commandStart = CommeoSensorTeachStart()
        commandStop = CommeoSensorTeachStop()
        commandResult = CommeoSensorTeachResult()

        commandStart.execute(self.gateway)
        commandResult.execute(self.gateway)
        while commandResult.timeLeft > 0:
            wait(commandResult.timeLeft+1)
            commandResult.execute(self.gateway)
        if commandResult.scanState == ScanState.END_SUCCESS:
            if commandResult.foundId != -1:
                return commandResult.foundId

    def getSensorValues(self, id):
        command = CommeoSensorGetValues(id)
        command.execute(self.gateway)
        return command

    def setSensorLabel(self, id, label):
        command = CommeoSensorSetLabel(id, label)
        command.execute(self.gateway)

    def deleteSensor(self, id):
        command = CommeoSensorDelete(id)
        command.execute(self.gateway)

    def setSensorManual(self, id, address, name):
        command = CommeoSensorWriteManual(id, address, name)
        command.execute(self.gateway)



from enum import Enum
from selve.communication import Command

from selve.protocol import MethodCall, ServiceState
from selve.protocol import ParameterType
from selve.protocol import DeviceType
from selve.protocol import CommandType
from selve.commands import Commands, CommeoCommandCommand, CommeoDeviceCommand, CommeoEventCommand, CommeoGroupCommand, CommeoParamCommand, CommeoSenSimCommand, CommeoSenderCommand, CommeoSensorCommand, CommeoServiceCommand
from selve.utils import singlemask
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
import logging



_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)

#######################
## Device Definition ##
#######################




from selve.device import Device

from selve.protocol import CommandTypeIveo, CommunicationType, DeviceClass, MethodCall, RepeaterState
from selve.protocol import ParameterType
from selve.protocol import DeviceType
from selve.commands import IveoCommand
from selve.communication import Command, CommandMask, CommandSingle
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
import logging


_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.DEBUG)

    
class IveoCommandSetRepeater(Command):
    def __init__(self, repeaterState):
        super().__init__(IveoCommand.SETREPEATER, [(ParameterType.INT, repeaterState)])

class IveoCommandGetRepeater(Command):
    def __init__(self):
        super().__init__(IveoCommand.GETREPEATER)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.repeaterState = RepeaterState(int(methodResponse.parameters[0][1]))

class IveoCommandFactory(CommandSingle):

    def __init__(self, iveoID):
        super().__init__(IveoCommand.FACTORY, iveoID)

class IveoCommandTeach(CommandSingle):
    def __init__(self, iveoID):
        super().__init__(IveoCommand.TEACH, iveoID)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])   

class IveoCommandLearn(CommandSingle):
    def __init__(self, iveoID):
        super().__init__(IveoCommand.LEARN, iveoID)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1]) 
         
class IveoCommandManual(CommandMask):
    def  __init__(self, mask, command):
        super().__init__(IveoCommand.MANUAL, mask, command)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1]) 

class IveoCommandAutomatic(CommandMask):
    def  __init__(self, mask, command):
        super().__init__(IveoCommand.AUTOMATIC, mask, command)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandResult(MethodCall):
    def __init__(self, command, mask, state):
        super().__init__(IveoCommand.RESULT, [(ParameterType.INT, command), (ParameterType.BASE64, mask), (ParameterType.INT, state)])

class IveoCommandSetLabel(Command):
    def __init__(self, iveoId, label):
        super().__init__(IveoCommand.SETLABEL, [(ParameterType.INT, iveoId), (ParameterType.STRING, label)])

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandSetConfig(Command):
    def __init__(self, iveoId, activity, device_type):
        super().__init__(IveoCommand.SETCONFIG, [(ParameterType.INT, iveoId), (ParameterType.INT, activity), (ParameterType.INT, device_type)])
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class IveoCommandGetConfig(CommandSingle):
    def __init__(self, iveoId):
        super().__init__(IveoCommand.GETCONFIG, iveoId)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.activity = methodResponse.parameters[2][1]
        self.deviceType = DeviceType(int(methodResponse.parameters[3][1]))

class IveoCommandGetIds(Command):
    def __init__(self):
        super().__init__(IveoCommand.GETIDS)
    
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)

class IveoDevice(Device):

    def __init__(self, gateway, iveoID, discover = False):
        super().__init__(gateway, iveoID, discover)
        self.communicationType = CommunicationType.IVEO
        self.deviceClass = DeviceClass.IVEO
        if discover:
            self.discover_properties()
    
    def executeCommand(self, commandType, automatic = False):
        if automatic:
            command = IveoCommandAutomatic(self.mask, commandType)
        else:
            command = IveoCommandManual(self.mask, commandType)
        command.execute(self.gateway)
        return command

    def discover_properties(self):
        try:
            command = IveoCommandGetConfig(self.ID)
            command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.activity = command.activity
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))

    def setRepeaterState(self, state):
        command = IveoCommandSetRepeater(state)
        command.execute(self.gateway)

    def getRepeaterState(self):
        command = IveoCommandGetRepeater()
        command.execute(self.gateway)
        return command

    def setIveoLabel(self):
        command = IveoCommandSetLabel(self.ID, self.name)
        command.execute(self.gateway)

    def setIveoConfig(self):
        command = IveoCommandSetConfig(self.ID, self.activity, self.device_type)
        command.execute(self.gateway)

    def getIveoConfig(self):
        command = IveoCommandGetConfig(self.ID)
        command.execute(self.gateway)
        self.device_type = command.deviceType
        self.name = command.name
        self.activity = command.activity

    def resetIveoChannel(self):
        command = IveoCommandFactory(self.ID)
        command.execute(self.gateway)
        self.gateway.deleteDevice(self.ID)

    def learnIveoChannel(self):
        command = IveoCommandLearn(self.ID)
        command.execute(self.gateway)

    def setToLearnMode(self):
        self.learnIveoChannel()        

    def teachIveoChannel(self, channel):
        command = IveoCommandTeach(channel)
        _LOGGER.info("Trying to teach channel " + str(channel))
        command.execute(self.gateway)
        if command.hasError:
            _LOGGER.info("Teaching failed for channel " + str(channel))
            return
        if command.executed:
            _LOGGER.info("Channel " + str(channel) + "successfully taught" )

    def manualIveoCommand(self, command):
        command = IveoCommandManual(self.mask, command)
        command.execute(self.gateway)
        return command

    def automaticIveoCommand(self, command):
        command = IveoCommandAutomatic(self.mask, command)
        command.execute(self.gateway)
        return command


    def stop(self):
        self.automaticIveoCommand(CommandTypeIveo.STOP)

    def moveDown(self):
        self.automaticIveoCommand(CommandTypeIveo.DRIVEDOWN)
    
    def moveUp(self):
        self.automaticIveoCommand(CommandTypeIveo.DRIVEUP)
    
    def moveIntermediatePosition1(self):
        self.manualIveoCommand(CommandTypeIveo.POSITION_1)

    def moveIntermediatePosition2(self):
        self.manualIveoCommand(CommandTypeIveo.POSITION_2)
    
    def deleteDevice(self):
        self.resetIveoChannel()

    def saveDevice(self):
        self.teachIveoChannel(self.ID)


from os import wait
from selve.communication import Command, CommandSingle
from selve.protocol import CommunicationType, DeviceClass, ScanState, TeachState, senderEvents
from selve.protocol import ParameterType
from selve.commands import CommeoSenderCommand
from selve.utils import true_in_list
from selve.utils import b64bytes_to_bitlist
from selve.device import Device
import logging
_LOGGER = logging.getLogger(__name__)


class CommeoSenderTeachStart(Command):
    def __init__(self):
        super().__init__(CommeoSenderCommand.TEACHSTART)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])

class CommeoSenderTeachStop(Command):
    def __init__(self):
        super().__init__(CommeoSenderCommand.TEACHSTOP)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
        
class CommeoSenderTeachResult(Command):
    def __init__(self):
        super().__init__(CommeoSenderCommand.TEACHRESULT)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = str(methodResponse.parameters[0][1])
        self.teachState = TeachState(int(methodResponse.parameters[1][1]))
        self.timeLeft = int(methodResponse.parameters[1][1])
        self.senderId = int(methodResponse.parameters[2][1])
        self.senderEvent = senderEvents(int(methodResponse.parameters[3][1]))

class CommeoSenderGetIDs(Command):
    def __init__(self):
        super().__init__(CommeoSenderCommand.GETIDS)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.ids = [ b for b in true_in_list(b64bytes_to_bitlist(methodResponse.parameters[0][1]))]
        _LOGGER.debug(self.ids)
        
class CommeoSenderGetInfo(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoSenderCommand.GETINFO, deviceId)

    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.name = methodResponse.parameters[0][1]
        self.rfAddress = methodResponse.parameters[2][1]
        self.rfChannel = int(methodResponse.parameters[3][1])
        self.rfResetCount = int(methodResponse.parameters[4][1])

class CommeoSenderGetValues(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoSenderCommand.GETVALUES, deviceId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.lastEvent = senderEvents(int(methodResponse.parameters[1][1]))
        
class CommeoSenderSetLabel(Command):
    def __init__(self, deviceId, name):
        super().__init__(CommeoSenderCommand.SETLABEL, [(ParameterType.INT, deviceId), (ParameterType.STRING, name)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
        
class CommeoSenderDelete(CommandSingle):
    def __init__(self, deviceId):
        super().__init__(CommeoSenderCommand.DELETE, deviceId)
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
        
class CommeoSenderWriteManual(Command):
    def __init__(self, deviceId, rfAddress, rfChannel, rfResetCount, name):
        super().__init__(CommeoSenderCommand.WRITEMANUAL, [(ParameterType.INT, deviceId), (ParameterType.INT, rfAddress), (ParameterType.INT, rfChannel), (ParameterType.INT, rfResetCount), (ParameterType.STRING, name)])
    def process_response(self, methodResponse):
        super().process_response(methodResponse)
        self.executed = bool(methodResponse.parameters[0][1])
        
class SenderDevice(Device):
    def __init__(self, gateway, id, discover = False):
        super().__init__(gateway, id, discover)
        self.communicationType = CommunicationType.COMMEO
        self.deviceClass = DeviceClass.SENDER
        if discover:
            self.discover_properties()

    def discover_properties(self):
        try:
            command = CommeoSenderGetInfo(self.ID)
            command.execute(self.gateway)
            self.device_type = command.deviceType
            self.name = command.name
            self.rfAddress = command.rfAddress
            self.state = command.state
            self.deviceClass = DeviceClass.SENDER
            self.communicationType = CommunicationType.COMMEO
        except Exception as e1:
            _LOGGER.exception ("not : " + str(e1))


    def scanSender(self):
        commandStart = CommeoSenderTeachStart()
        commandStop = CommeoSenderTeachStop()
        commandResult = CommeoSenderTeachResult()

        commandStart.execute(self.gateway)
        commandResult.execute(self.gateway)
        while commandResult.timeLeft > 0:
            wait(commandResult.timeLeft+1)
            commandResult.execute(self.gateway)
        if commandResult.scanState == ScanState.END_SUCCESS:
            if commandResult.foundId != -1:
                return commandResult.foundId
                
    def getDeviceValues(self, id):
        command = CommeoSenderGetValues(id)
        command.execute(self.gateway)
        return command

    def setDeviceLabel(self, id, label):
        command = CommeoSenderSetLabel(id, label)
        command.execute(self.gateway)

    def deleteDevice(self, id):
        command = CommeoSenderDelete(id)
        command.execute(self.gateway)

    def setDeviceManual(self, id, address, channel, resetCount, name):
        command = CommeoSenderWriteManual(id, address, channel, resetCount, name)
        command.execute(self.gateway)



from selve.utils import *
from selve.communication import *
import logging
_LOGGER = logging.getLogger(__name__)

class Group():
    ## A group consists of multiple devices. We have to treat it like a device.

    def __init__(self, gateway, ID, communicationType, discover = False):
        self.ID = ID
        self.gateway = gateway
        self.mask = singlemask(ID)
        self.device_type = DeviceType.UNKNOWN ## Device type according to devices in group
        self.communicationType = communicationType
        self.name = "Not defined"
        if discover:
            self.discover_properties()
    
    def executeCommand(self, commandType, automatic = False):
        if automatic:
            command = CommeoDeviceCommand(self.mask, commandType)
        else:
            command = CommeoDeviceCommand(self.mask, commandType)
        command.execute(self.gateway)
        return command

    # def discover_properties(self):
    #     command = CommeoDeviceGetValues(self.ID)
    #     command.execute(self.gateway)
    #     self.device_type = command.deviceType
    #     self.name = command.name

    ## Actor ##


    ## Sensor ##


    ## SenSim ##


    ## Sender ##


    ## Iveo ##


    

    
    def __str__(self):
        return "Device " + self.device_type.name + " of type: " + self.communicationType + " on channel " + str(self.ID) + " with name " + self.name